@@ -28,12 +28,18 @@
 #include <libanjuta/anjuta-debug.h>
 
 #include "plugin.h"
+#include <glib.h>
+#include <glib/gi18n.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 
 #include "debugger.h"
 
 #include <libanjuta/interfaces/ianjuta-debugger.h>
 #include <libanjuta/interfaces/ianjuta-debugger-breakpoint.h>
-#include <libanjuta/interfaces/ianjuta-cpu-debugger.h>
+#include <libanjuta/interfaces/ianjuta-debugger-register.h>
+#include <libanjuta/interfaces/ianjuta-debugger-memory.h>
+#include <libanjuta/interfaces/ianjuta-debugger-instruction.h>
 #include <libanjuta/interfaces/ianjuta-debugger-variable.h>
 #include <libanjuta/interfaces/ianjuta-terminal.h>
 #include <libanjuta/anjuta-plugin.h>
@@ -68,10 +74,12 @@ struct _GdbPluginClass
 /* Terminal functions
  *---------------------------------------------------------------------------*/
 
+#define PREF_TERMINAL_COMMAND "anjuta.command.terminal"
+
 static void
 gdb_plugin_stop_terminal (GdbPlugin* plugin)
 {
-	DEBUG_PRINT ("In function: gdb_plugin_stop_terminal()");
+	DEBUG_PRINT ("%s", "In function: gdb_plugin_stop_terminal()");
 
 	if (plugin->term_pid > 0) {
 		kill (plugin->term_pid, SIGTERM);
@@ -83,7 +91,7 @@ static gchar*
 gdb_plugin_start_terminal (GdbPlugin* plugin)
 {
 	gchar *file, *cmd;
-       	gchar *tty = NULL;
+	gchar *tty = NULL;
 	IAnjutaTerminal *term;
 
 	DEBUG_PRINT ("In function: gdb_plugin_start_terminal() previous pid %d", plugin->term_pid);
@@ -91,17 +99,6 @@ gdb_plugin_start_terminal (GdbPlugin* plugin)
 	/* Close previous terminal if needed */
 	gdb_plugin_stop_terminal (plugin);
 
-	/* Get terminal plugin */	
-	term = anjuta_shell_get_interface (ANJUTA_PLUGIN (plugin)->shell, IAnjutaTerminal, NULL);
-	if (term == NULL)
-	{
-
-		anjuta_util_dialog_error (GTK_WINDOW (ANJUTA_PLUGIN (plugin)->shell),
-					  _("Anjuta terminal plugin is not installed. The program will be run without a terminal."));
-
-		return NULL;
-	}
-
 	/* Check if anjuta launcher is here */
 	if (anjuta_util_prog_is_installed ("anjuta_launcher", TRUE) == FALSE)
 	{
@@ -112,7 +109,7 @@ gdb_plugin_start_terminal (GdbPlugin* plugin)
 	if (mkfifo (file, 0664) < 0)
 	{
 		anjuta_util_dialog_error (GTK_WINDOW (ANJUTA_PLUGIN (plugin)->shell),
-					  _("Fail to create fifo file named %s. The program will be run without a terminal"), file);
+					  _("Failed to create fifo file named %s. The program will run without a terminal."), file);
 		g_free (file);
 
 		return NULL;
@@ -120,8 +117,59 @@ gdb_plugin_start_terminal (GdbPlugin* plugin)
 
 	/* Launch terminal */
 	cmd = g_strconcat ("anjuta_launcher --__debug_terminal ", file, NULL);
-	plugin->term_pid = ianjuta_terminal_execute_command (term, NULL, cmd, NULL);
-	g_free (cmd);
+	
+	/* Get terminal plugin */	
+	term = anjuta_shell_get_interface (ANJUTA_PLUGIN (plugin)->shell, IAnjutaTerminal, NULL);
+	if (term == NULL)
+	{
+		/* Use gnome terminal or another user defined one */
+		AnjutaPreferences *pref;
+		gchar *term_cmd;
+		gchar **argv;
+		
+		pref = anjuta_shell_get_preferences (ANJUTA_PLUGIN (plugin)->shell, NULL);
+		term_cmd = anjuta_preferences_get (pref, PREF_TERMINAL_COMMAND);
+		if (!term_cmd)
+		{
+			term_cmd = g_strdup ("gnome-terminal --disable-factory -e %s");
+		}
+		if (g_shell_parse_argv (term_cmd, NULL, &argv, NULL))
+		{
+			GPid gpid;
+			gchar **arg;
+			
+			/* Replace %s by command */
+			for (arg = argv; *arg != NULL; arg++)
+			{
+				if (strcmp(*arg, "%s") == 0)
+				{
+					g_free (*arg);
+					*arg = cmd;
+				}
+			}
+			
+			if (g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, &gpid, NULL))
+			{
+				plugin->term_pid = gpid;
+			}
+			else
+			{
+				plugin->term_pid = -1;
+			}
+			g_strfreev (argv);
+		}
+		else
+		{
+			plugin->term_pid = -1;
+		}
+		g_free (term_cmd);
+	}
+	else
+	{
+		/* Use Anjuta terminal plugin */
+		plugin->term_pid = ianjuta_terminal_execute_command (term, NULL, cmd, NULL, NULL);
+		g_free (cmd);
+	}
 
 	if (plugin->term_pid > 0)
 	{
@@ -196,7 +244,7 @@ gdb_plugin_activate_plugin (AnjutaPlugin* plugin)
 {
 	/* GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin); */
 
-	DEBUG_PRINT ("GDB: Activating Gdb plugin...");
+	DEBUG_PRINT ("%s", "GDB: Activating Gdb plugin...");
 
 	return TRUE;
 }
@@ -206,7 +254,7 @@ gdb_plugin_deactivate_plugin (AnjutaPlugin* plugin)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 
-	DEBUG_PRINT ("GDB: Deactivating Gdb plugin...");
+	DEBUG_PRINT ("%s", "GDB: Deactivating Gdb plugin...");
 
 	if (this->debugger != NULL)
 	{
@@ -251,7 +299,7 @@ gdb_plugin_dispose (GObject* obj)
 		debugger_free (this->debugger);
 		this->debugger = NULL;
 	}
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, dispose, (G_OBJECT (obj)));
+	G_OBJECT_CLASS (parent_class)->dispose (obj);
 }
 
 /* finalize is the last destruction step. It must free all memory allocated
@@ -261,7 +309,7 @@ gdb_plugin_dispose (GObject* obj)
 static void
 gdb_plugin_finalize (GObject* obj)
 {
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (G_OBJECT (obj)));
+	G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
 
 /* class_init intialize the class itself not the instance */
@@ -310,7 +358,8 @@ idebugger_load (IAnjutaDebugger *plugin, const gchar *file, const gchar* mime_ty
 	{
 		/* Hope that the target is supported */
 	}
-	else if (strcmp (mime_type, "application/x-executable") == 0)
+	else if ((strcmp (mime_type, "application/x-executable") == 0) ||
+		(strcmp (mime_type, "application/octet-stream") == 0))
 	{
 		/* Supported target */
 	}
@@ -347,6 +396,26 @@ idebugger_unload (IAnjutaDebugger *plugin, GError **err)
 	return TRUE;
 }
 
+static gboolean
+idebugger_set_working_directory (IAnjutaDebugger *plugin, const gchar *directory, GError **err)
+{
+	GdbPlugin *self = ANJUTA_PLUGIN_GDB (plugin);
+
+	debugger_set_working_directory (self->debugger, directory);
+
+	return TRUE;
+}
+
+static gboolean
+idebugger_set_environment (IAnjutaDebugger *plugin, gchar **variables, GError **err)
+{
+	GdbPlugin *self = ANJUTA_PLUGIN_GDB (plugin);
+
+	debugger_set_environment (self->debugger, variables);
+
+	return TRUE;
+}
+
 static gboolean
 idebugger_attach (IAnjutaDebugger *plugin, pid_t pid, const GList *search_dirs, GError **err)
 {
@@ -367,7 +436,20 @@ idebugger_start (IAnjutaDebugger *plugin, const gchar *argument, gboolean termin
 	gchar *tty;
 
 	tty = terminal ? gdb_plugin_start_terminal (this) : NULL;
-	debugger_start_program (this->debugger, argument, tty, stop);
+	debugger_start_program (this->debugger, NULL, argument, tty, stop);
+	g_free (tty);
+
+	return TRUE;
+}
+
+static gboolean
+idebugger_connect (IAnjutaDebugger *plugin, const gchar *server, const gchar *argument, gboolean terminal, gboolean stop, GError **err)
+{
+	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
+	gchar *tty;
+
+	tty = terminal ? gdb_plugin_start_terminal (this) : NULL;
+	debugger_start_program (this->debugger, server, argument, tty, stop);
 	g_free (tty);
 
 	return TRUE;
@@ -380,7 +462,7 @@ idebugger_quit (IAnjutaDebugger *plugin, GError **err)
 
 	if (!debugger_stop (this->debugger))
 	{
-		DEBUG_PRINT ("set error");
+		DEBUG_PRINT ("%s", "set error");
 		g_set_error (err, IANJUTA_DEBUGGER_ERROR, IANJUTA_DEBUGGER_CANCEL, "Command cancelled by user");
 
 		return FALSE;
@@ -396,7 +478,7 @@ idebugger_abort (IAnjutaDebugger *plugin, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 
-	DEBUG_PRINT ("idebugger abort\n");
+	DEBUG_PRINT ("%s", "idebugger abort\n");
 	if (!debugger_abort (this->debugger))
 	{
 		g_set_error (err, IANJUTA_DEBUGGER_ERROR, IANJUTA_DEBUGGER_CANCEL, "Command cancelled by user");
@@ -440,7 +522,7 @@ idebugger_step_over (IAnjutaDebugger *plugin, GError **err)
 }
 
 static gboolean
-idebugger_run_to (IAnjutaDebugger *plugin, const gchar *file,
+idebugger_run_to (IAnjutaDebugger *plugin, const gchar* file,
 						   gint line, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
@@ -731,7 +813,10 @@ idebugger_iface_init (IAnjutaDebuggerIface *iface)
 	iface->get_state = idebugger_get_state;
 	iface->attach = idebugger_attach;
 	iface->load = idebugger_load;
+	iface->set_working_directory = idebugger_set_working_directory;
+	iface->set_environment = idebugger_set_environment;
 	iface->start = idebugger_start;
+	iface->connect = idebugger_connect;
 	iface->unload = idebugger_unload;
 	iface->quit = idebugger_quit;
 	iface->abort = idebugger_abort;
@@ -809,7 +894,7 @@ idebugger_breakpoint_add_at_function (IAnjutaDebuggerBreakpoint *plugin, const g
 }
 
 static gboolean
-idebugger_breakpoint_add_at_address (IAnjutaDebuggerBreakpoint *plugin, guint address, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
+idebugger_breakpoint_add_at_address (IAnjutaDebuggerBreakpoint *plugin, gulong address, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 	
@@ -871,22 +956,22 @@ idebugger_breakpoint_list (IAnjutaDebuggerBreakpoint *plugin, IAnjutaDebuggerCal
 static void
 idebugger_breakpoint_iface_init (IAnjutaDebuggerBreakpointIface *iface)
 {
-	iface->implement = idebugger_breakpoint_implement;
-	iface->set_at_line = idebugger_breakpoint_add_at_line;
-	iface->clear = idebugger_breakpoint_remove;
-	iface->list = idebugger_breakpoint_list;
-	iface->set_at_address = idebugger_breakpoint_add_at_address;
-	iface->set_at_function = idebugger_breakpoint_add_at_function;
-	iface->enable = idebugger_breakpoint_enable;
-	iface->ignore = idebugger_breakpoint_ignore;
-	iface->condition = idebugger_breakpoint_condition;
+	iface->implement_breakpoint = idebugger_breakpoint_implement;
+	iface->set_breakpoint_at_line = idebugger_breakpoint_add_at_line;
+	iface->clear_breakpoint = idebugger_breakpoint_remove;
+	iface->list_breakpoint = idebugger_breakpoint_list;
+	iface->set_breakpoint_at_address = idebugger_breakpoint_add_at_address;
+	iface->set_breakpoint_at_function = idebugger_breakpoint_add_at_function;
+	iface->enable_breakpoint = idebugger_breakpoint_enable;
+	iface->ignore_breakpoint = idebugger_breakpoint_ignore;
+	iface->condition_breakpoint = idebugger_breakpoint_condition;
 }
 
-/* Implementation of IAnjutaCpuDebugger interface
+/* Implementation of IAnjutaDebuggerRegister interface
  *---------------------------------------------------------------------------*/
 
 static gboolean
-icpu_debugger_list_register (IAnjutaCpuDebugger *plugin, IAnjutaDebuggerCallback callback , gpointer user_data, GError **err)
+idebugger_register_list (IAnjutaDebuggerRegister *plugin, IAnjutaDebuggerCallback callback , gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 
@@ -896,7 +981,7 @@ icpu_debugger_list_register (IAnjutaCpuDebugger *plugin, IAnjutaDebuggerCallback
 }
 
 static gboolean
-icpu_debugger_update_register (IAnjutaCpuDebugger *plugin, IAnjutaDebuggerCallback callback , gpointer user_data, GError **err)
+idebugger_register_update (IAnjutaDebuggerRegister *plugin, IAnjutaDebuggerCallback callback , gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 
@@ -906,7 +991,7 @@ icpu_debugger_update_register (IAnjutaCpuDebugger *plugin, IAnjutaDebuggerCallba
 }
 
 static gboolean
-icpu_debugger_write_register (IAnjutaCpuDebugger *plugin, IAnjutaDebuggerRegister *value, GError **err)
+idebugger_register_write (IAnjutaDebuggerRegister *plugin, IAnjutaDebuggerRegisterData *value, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 
@@ -915,8 +1000,19 @@ icpu_debugger_write_register (IAnjutaCpuDebugger *plugin, IAnjutaDebuggerRegiste
 	return TRUE;
 }
 
+static void
+idebugger_register_iface_init (IAnjutaDebuggerRegisterIface *iface)
+{
+	iface->list_register = idebugger_register_list;
+	iface->update_register = idebugger_register_update;
+	iface->write_register = idebugger_register_write;
+}
+
+/* Implementation of IAnjutaDebuggerMemory interface
+ *---------------------------------------------------------------------------*/
+
 static gboolean
-icpu_debugger_inspect_memory (IAnjutaCpuDebugger *plugin, guint address, guint length, IAnjutaDebuggerCallback callback , gpointer user_data, GError **err)
+idebugger_memory_inspect (IAnjutaDebuggerMemory *plugin, gulong address, guint length, IAnjutaDebuggerCallback callback , gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 
@@ -925,8 +1021,17 @@ icpu_debugger_inspect_memory (IAnjutaCpuDebugger *plugin, guint address, guint l
 	return TRUE;
 }
 
+static void
+idebugger_memory_iface_init (IAnjutaDebuggerMemoryIface *iface)
+{
+	iface->inspect = idebugger_memory_inspect;
+}
+
+/* Implementation of IAnjutaDebuggerInstruction interface
+ *---------------------------------------------------------------------------*/
+
 static gboolean
-icpu_debugger_disassemble (IAnjutaCpuDebugger *plugin, guint address, guint length, IAnjutaDebuggerCallback callback , gpointer user_data, GError **err)
+idebugger_instruction_disassemble (IAnjutaDebuggerInstruction *plugin, gulong address, guint length, IAnjutaDebuggerCallback callback , gpointer user_data, GError **err)
 {
 	GdbPlugin *this = (GdbPlugin *)plugin;
 
@@ -936,7 +1041,7 @@ icpu_debugger_disassemble (IAnjutaCpuDebugger *plugin, guint address, guint leng
 }
 
 static gboolean
-icpu_debugger_stepi_in (IAnjutaCpuDebugger *plugin, GError **err)
+idebugger_instruction_step_in (IAnjutaDebuggerInstruction *plugin, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 	
@@ -946,7 +1051,7 @@ icpu_debugger_stepi_in (IAnjutaCpuDebugger *plugin, GError **err)
 }
 
 static gboolean
-icpu_debugger_stepi_over (IAnjutaCpuDebugger *plugin, GError **err)
+idebugger_instruction_step_over (IAnjutaDebuggerInstruction *plugin, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 	
@@ -956,7 +1061,7 @@ icpu_debugger_stepi_over (IAnjutaCpuDebugger *plugin, GError **err)
 }
 
 static gboolean
-icpu_debugger_run_to_address (IAnjutaCpuDebugger *plugin, guint address, GError **err)
+idebugger_instruction_run_to_address (IAnjutaDebuggerInstruction *plugin, gulong address, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 	
@@ -966,16 +1071,12 @@ icpu_debugger_run_to_address (IAnjutaCpuDebugger *plugin, guint address, GError
 }
 
 static void
-icpu_debugger_iface_init (IAnjutaCpuDebuggerIface *iface)
+idebugger_instruction_iface_init (IAnjutaDebuggerInstructionIface *iface)
 {
-	iface->list_register = icpu_debugger_list_register;
-	iface->update_register = icpu_debugger_update_register;
-	iface->write_register = icpu_debugger_write_register;
-	iface->inspect_memory = icpu_debugger_inspect_memory;
-	iface->disassemble = icpu_debugger_disassemble;
-	iface->stepi_in = icpu_debugger_stepi_in;
-	iface->stepi_over = icpu_debugger_stepi_over;
-	iface->run_to_address = icpu_debugger_run_to_address;
+	iface->disassemble = idebugger_instruction_disassemble;
+	iface->step_in_instruction = idebugger_instruction_step_in;
+	iface->step_over_instruction = idebugger_instruction_step_over;
+	iface->run_to_address = idebugger_instruction_run_to_address;
 }
 
 /* Implementation of IAnjutaDebuggerVariable interface
@@ -1055,7 +1156,9 @@ idebugger_variable_iface_init (IAnjutaDebuggerVariableIface *iface)
 ANJUTA_PLUGIN_BEGIN (GdbPlugin, gdb_plugin);
 ANJUTA_PLUGIN_ADD_INTERFACE(idebugger, IANJUTA_TYPE_DEBUGGER);
 ANJUTA_PLUGIN_ADD_INTERFACE(idebugger_breakpoint, IANJUTA_TYPE_DEBUGGER_BREAKPOINT);
-ANJUTA_PLUGIN_ADD_INTERFACE(icpu_debugger, IANJUTA_TYPE_CPU_DEBUGGER);
+ANJUTA_PLUGIN_ADD_INTERFACE(idebugger_register, IANJUTA_TYPE_DEBUGGER_REGISTER);
+ANJUTA_PLUGIN_ADD_INTERFACE(idebugger_memory, IANJUTA_TYPE_DEBUGGER_MEMORY);
+ANJUTA_PLUGIN_ADD_INTERFACE(idebugger_instruction, IANJUTA_TYPE_DEBUGGER_INSTRUCTION);
 ANJUTA_PLUGIN_ADD_INTERFACE(idebugger_variable, IANJUTA_TYPE_DEBUGGER_VARIABLE);
 ANJUTA_PLUGIN_END;
 
